#!/usr/bin/python3
# vim: expandtab

# -*- coding: UTF-8 -*-

# Copyright 2006-2007 (C) Raster Software Vigo (Sergio Costas)
# Copyright 2009-2014 (C) Rick Richardson
#
# gmd-applet-3:
#       This file is part of Gnome Manual Duplex (GMD).  It is for gnome-3.
#
# GMD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# GMD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import time
import sys

if sys.version > '3':
    python2 = os.popen('which python2 2> /dev/null').read().rstrip()
    if python2:
        args = sys.argv[:]
        args.insert(0,python2)
        os.execv(python2,args)
    else:
        sys.exit("%s requires Python Version 2 (python2 not in PATH)" \
            % os.path.basename(__file__))

import select
import subprocess
import pwd
import shutil

needed = ""

try:
    import gamin
except :
    needed += "python-gamin\n"

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GObject, Gtk, GdkPixbuf, Gdk

def create_dir(directory, mode):
    orig_umask = os.umask(0)
    try:
        os.mkdir(directory, mode)
    except OSError:
        return
    finally:
       os.umask(orig_umask)

def send_error(message):
    orig_umask = os.umask(0)
    try:
        if not os.path.exists("/var/tmp/gmd"):
            os.mkdir("/var/tmp/gmd", 0o1777)
        # try to create the file with write permissions for everyone
        fp = os.fdopen(os.open("/var/tmp/gmd/error.log", os.O_WRONLY | os.O_APPEND | os.O_CREAT, 0o666), "w")
    except:
        return
    finally:
        os.umask(orig_umask)
    fp.write(str(pwd.getpwuid(os.getuid())[0]) + ": " + message + "\n")
    fp.close()

def read_line(fp):
    line = fp.readline()
    if line[-1] == '\n':
        line = line[:-1]
    return line

def launch_gmd(filename):
    global directory

    time.sleep(1) # wait one second to ensure that the file is fully wrote
    try:
        fp = open(directory + "/list/" + filename, "r");
    except IOError:
        send_error("Error de E/S")
        return
    newfile = read_line(fp)
    if (-1 != newfile.find("/")):
        # don't allow the '/' in the filename to avoid security issues
        send_error( "Has /: " + newfile)
        return
    print(directory + "/" + newfile)
    title2 = read_line(fp)
    
    title = ""
    for letter in title2:
        if letter == '"':
            title += "'"
        elif letter == '\\':
            title += '/'
        else:
            title += letter
    print(title)
    copies = read_line(fp)
    fp.close()
    print(copies)
    
    #command = '/usr/bin/gnome-manual-duplex -T "' + title + '" -# ' + copies \
    #   + ' "' + directory + "/" + newfile + '"'
    command = '/usr/bin/gnome-manual-duplex "' + directory + "/" + newfile + '"'
    send_error(command)
    p = subprocess.Popen(command, shell=True, bufsize=32768)
    p.wait()
    
    command='rm -f "' + directory + "/" + newfile + '" "' + directory \
                + "/list/" + filename + '"'
    p = subprocess.Popen(command, shell=True, bufsize=32768)
    p.wait()

def read_event2(path, event):
    print("path: " + str(path) + " event: " + str(event))
    if (event == 5): # new file created
        launch_gmd(path)
    return True

def read_event(path, event):
    global fc
    
    print("path: " + str(path) + " event: " + str(event))

    ret = fc.event_pending()
    if ret > 0:
        fc.handle_one_event()
        fc.handle_events()
    return True

def init_scan():
    global fc
    global basedir
    global directory
    global event0
    global request

    basedir = "/var/tmp/gmd"
    try:
        directory = basedir + "/" + str(pwd.getpwuid(os.getuid())[0])
    except :
        send_error("Failed to get the UID")
        return Gtk.FALSE

    # erase the directory to ensure that there are no spare works
    shutil.rmtree(directory, True)

    # create base directory, if required, with write permissions for all and "sticky" bit like /tmp
    create_dir(basedir, 0o1777)
    # set priviliges to RWX-WX-WX, to allow everybody to write into it, but
    # only the user to read, to preserve privacy
    create_dir(directory, 0o733)
    create_dir(directory + "/list", 0o733)

    fc = gamin.WatchMonitor()
    
    if (fc == None):
        send_error("Failed to create a GAMIN session. "
                    + "Check that the GAMIN daemon is running\n")
    
    request = fc.watch_directory(directory + "/list", read_event2)
            
    if (request == None):
        send_error("Failed to create a GAMIN request. "
                    + "Check that the GAMIN daemon is running\n")

    event0 = GObject.io_add_watch(fc.get_fd(), GObject.IO_IN, read_event)
    
    if (event0 == None):
        send_error("Failed to create a FAM event. "
                    + "Check that the FAM daemon is running\n")


def wdelete_event(widget, event, data=None):
    return False
        
def wdestroy(widget, data=None):
    sys.exit(0)

def factory(applet, iid):
    event_box = Gtk.EventBox()
    applet.add(event_box)
    event_box.show()

    image = Gtk.Image()
    pixbuf = GdkPixbuf.Pixbuf.new_from_file(
        "/usr/share/pixmaps/gnome-manual-duplex.png")
    #w,h = Gtk.icon_size_lookup(Gtk.IconSize.BUTTON)
    w = 28; h = 28
    scaled = pixbuf.scale_simple(w, h, GdkPixbuf.InterpType.BILINEAR)
    image.set_from_pixbuf(scaled)
    image.set_tooltip_text("Gnome Manual Duplex Applet")
    event_box.add(image)
    image.show()
    event_box.connect("button-press-event", show_menu, applet)
    applet.show_all()
    check_needed()

def applet_factory(applet, iid, data):
    factory(applet, iid)

def add_systray_icon(main_window):
    #assert(Gtk.gtk_version > (2, 10))

    class GmdStatusIcon(Gtk.StatusIcon):
        def __init__(self):
                GObject.GObject.__init__(self)
                menu = '''
                    <ui>
                     <menubar name="Menubar">
                      <menu action="Menu">
                       <menuitem action="About"/>
                       <menuitem action="Quit"/>
                      </menu>
                     </menubar>
                    </ui>
                '''
                self.mainwindow = main_window
                actions = [ ('Menu',  None, 'Menu'),
                            ('About', Gtk.STOCK_ABOUT,
                              '_About...', None, 'About Manual Duplex Applet', self.on_about),
                            ('Quit', Gtk.STOCK_QUIT,
                              '_Quit', None, '', self.on_quit)]
                ag = Gtk.ActionGroup('Actions')
                ag.add_actions(actions)
                self.manager = Gtk.UIManager()
                self.manager.insert_action_group(ag, 0)
                self.manager.add_ui_from_string(menu)
                self.menu = self.manager.get_widget('/Menubar/Menu/About').props.parent
                self.set_from_file("/usr/share/pixmaps/gnome-manual-duplex.png")
                #self.set_tooltip("Manual Duplex Applet")
                self.set_visible(True)
                self.connect('activate', self.on_activate)
                self.connect('popup-menu', self.on_popup_menu)

        def on_activate(self, data):
            os.system("gnome-manual-duplex")

        def on_popup_menu(self, status, button, time):
                self.menu.popup(None, None, None, None, button, time)

        def on_about(self, data):
            showAboutDialog(data)
            
        def on_quit(self, data):
            Gtk.main_quit()
 
    GmdStatusIcon()
    check_needed()

def create_menu (applet):
    actions = [("About", Gtk.STOCK_ABOUT, "_About", None, None, showAboutDialog)]
    actiongroup = Gtk.ActionGroup.new("GMDAppletActions")
    actiongroup.add_actions(actions, None)
    xml = """
            <menuitem name="About" action="About" />
            """
    applet.setup_menu(xml, actiongroup)

def showAboutDialog(self, *arguments, **keywords):
    about = Gtk.AboutDialog()
    about.set_name("GMD")
    about.set_program_name("gmd-applet.py")
    about.set_version("v0.70")
    about.set_copyright(
        "Copyright 2009-2014 Rick Richardson. All Rights Reserved.")
    about.set_license("GPL v2")
    about.set_website("http://sourceforge.net/projects/g-manual-duplex")
    about.set_authors( ["Rick Richardson", "Sergio Costas"] )
    about.set_icon_name('printer')
    about.set_logo(None)
    about.run()
    about.destroy()

def show_menu(widget, event, applet):
    if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 3:
        widget.emit_stop_by_name("button_press_event")
        create_menu(applet)
    if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
        os.system("gnome-manual-duplex")

def check_needed():
    global needed

    os.system(
        'lpadmin -p GnomeManualDuplex -m gnome-manual-duplex.ppd -E -v gmd:/ -L "Virtual Printer"')

    if needed == "":
        init_scan()
    else:
        window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
        window.set_title("Error, module not found")
        window.connect("delete_event", wdelete_event)
        window.connect("destroy", wdestroy)
        window.set_border_width(10)
        button = Gtk.Button("Ok")
        button.connect("clicked", wdestroy, None)
        label = Gtk.Label(label="You need to install the following python modules in order to use GMD:\n\n" + needed)
        box = Gtk.VBox()
        window.add(box)
        box.add(label)
        box.add(button)
        button.show()
        label.show()
        box.show()
        window.show()
    
    return True

fc = None
request = None
event0 = None
directory = ""
window = None

if (len(sys.argv) == 2) and (sys.argv[1] == "standalone"):
    check_needed()
    Gtk.main()
elif (len(sys.argv) == 2) and (sys.argv[1] == "run-in-window"):
    main_window = Gtk.Window(type=Gtk.WindowType.TOPLEVEL)
    main_window.set_title("Python Applet")
    main_window.connect("destroy", Gtk.main_quit)
    app = gnomeapplet.Applet()
    factory(app, None)
    app.reparent(main_window)
    main_window.show_all()
    Gtk.main()
    sys.exit()
elif (len(sys.argv) == 2) and (sys.argv[1] == "run-in-tray"):
    # should be POPUP so window manager won't list it
    main_window = Gtk.Window(type=Gtk.WindowType.POPUP)
    main_window.set_title("Python Applet")
    main_window.connect("destroy", Gtk.main_quit)
    add_systray_icon(main_window)
    main_window.iconify()
    # Removed due to POPUP...
    # main_window.show_all() 
    Gtk.main()
    sys.exit()
elif True:
    from gi.repository import PanelApplet
    PanelApplet.Applet.factory_main(
                "GnomeManualDuplexAppletFactory",  # id
                PanelApplet.Applet.__gtype__,   # gtype
                applet_factory,                 # factory callback
                None)                           # factory data pointer
else:
    gnomeapplet.bonobo_factory("OAFIID:GNOME_GMD_applet_Factory",
        gnomeapplet.Applet.__gtype__, "GMD", "3", factory)
